# Copyright (C) 2019 Heritage Observatory.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Heritage Observatory or one of its subsidiaries.  You shall not
# disclose this confidential information and shall use it only in
# accordance with the terms of the license agreement or other applicable
# agreement you entered into with Heritage Observatory.
#
# HERITAGE OBSERVATORY MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
# SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT
# NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
# A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  HERITAGE OBSERVATORY SHALL
# NOT BE LIABLE FOR ANY LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A
# RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
# DERIVATIVES.

from majormode.perseus.model.enum import Enum

# Occupancy of an heritage (excluding places of worship).
HeritageOccupancy = Enum(
    'vacant',
    'part_occupied',
    'occupied',
    'unknown',
    'n_a'
)

HeritageOwnerType = Enum(
    'local_authority',
    'private',
    'religious_organization',
    'multiple_owners'
)

# Priority for action is assessed on a scale of `A` to `F`, where `A` is
# the highest priority for a site which is deteriorating rapidly with no
# solution to secure its future, and `F` is the lowest priority.
#
# For buildings and structures and places of worship the following
# priority categories are used as an indication of trend and as a means
# of prioritising action:
HeritageAtRiskActionPriority = Enum(
    # Immediate risk of further rapid deterioration or loss of fabric; no
    # solution agreed.
    'A',

    # Immediate risk of further rapid deterioration or loss of fabric;
    # solution agreed but not yet implemented.
    'B',

    # Slow decay; no solution agreed.
    'C',

    # Slow decay; solution agreed but not yet implemented.
    'D',

    # Under repair or in fair to good repair, but no user identified; or
    # under threat of vacancy with no obvious new user (applicable only to
    # buildings capable of beneficial use).
    'E',

    # Repair scheme in progress and (where applicable) end use or user
    # identified; functionally redundant buildings with new use agreed but
    # not yet implemented.
    'F'
)

# Condition for buildings, including places of worship.
#
# For sites that cover areas (scheduled monuments (archaeology
# assessments) and parks and gardens) one overall condition category is
# recorded. The category may relate only to the part of the site or
# monument that is at risk and not the whole site.
#
# If a site has suffered from heritage crime it is noted in the summary.
# Heritage crime is defined as any offence which harms the heritage
# asset or its setting and includes arson, graffiti, lead theft and
# vandalism.
HeritageCondition = Enum(
    # Extensive significant problems.
    #
    # A building where there has been structural failure or where there are
    # clear signs of structural instability; where applicable, if there has
    # been loss of significant areas of roof covering, leading to major
    # deterioration of the interior; or where there has been a major fire or
    # other disaster affecting most of the building. The classification will
    # include derelict buildings and would include buildings which are
    # incomplete.
    'bad',

    # Generally unsatisfactory with major localised problems.
    #
    # A building or structure where there has been general deterioration of
    # the building fabric. There will obviously be faults likely to lead to
    # structural failure, such as an area of missing tiles though the roof
    # structure appears to be sound, and/or deterioration masonry or timber
    # frame, and/or defective rainwater goods. A building with numerous
    # failings would fall into this class. This classification could include
    # a building where there has been a fire or other disaster.
    'poor',

    # Generally satisfactory but with significant localised problems.
    #
    # A building which is structurally sound but in need of minor repair, or
    # showing lack of general maintenance e.g. decayed window frames or
    # gutters blocked or signs of water ingress.
    'fair',

    # Generally satisfactory but with minor.
    #
    # Structurally sound, water-tight with no significant repairs needed.
    'good'
)

# Principal designation of an heritage.
HeritageDesignation = Enum(
    # Listed Building and Structure (LB): When a building is recognised as
    # being of special architectural or historic interest it is added to the
    # statutory list. Structures that might not be classified as
    # 'buildings' ­ such as railings, gate piers, walls, war memorials,
    # gravestones, post boxes and telephone boxes ­ can all be Listed
    # Buildings.
    'LB',

    # Listed Place of Worship (LPW).
    'LPW',

    # Scheduled Monument (SM), Archaeological Site or Historic Building:
    # Historic buildings and archaeological sites of national importance are
    # given legal protection by being placed on a 'Schedule' of monuments.
    # Examples of Scheduled Monuments are Roman remains, burial mounds,
    # castles, bridges, earthworks, the remains of deserted villages, and
    # industrial sites. Scheduled Monuments can not include ecclesiastical
    # or residential buildings (except for associated caretaker’s dwellings),
    # and unlike Listed Buildings they are not assigned grades.
    'SM',

    # Registered Park and Garden (RPG): Planned open spaces, including
    # public parks, cemeteries, the grounds of private houses, and town
    # squares.
    'RGP',

    # Registered Battlefield (RB).
    'RB',

    # Protected Wreck Site (PWS).
    'PWS',

    # Conservation Area (CA).
    'CA'
)

# Buildings are given one of three grades which denote their level of
# importance, Grade I being the highest and Grade II the lowest.
HeritageGrade = Enum(
    # Grade I: Buildings of exceptional interest.
    'I',

    # Grade II*: Particularly important buildings of more than special
    # interest.
    'II*',

    # Grade II: buildings that are of special interest, warranting every
    # effort to preserve them
    'II'
)


HeritageAtRiskReportStatus = Enum(
    'draft',
    'deleted',
    'disabled',
    'enabled',
    'pending',
)



# Copyright (C) 2019 Heritage Observatory.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Heritage Observatory or one of its subsidiaries.  You shall not
# disclose this confidential information and shall use it only in
# accordance with the terms of the license agreement or other applicable
# agreement you entered into with Heritage Observatory.
#
# HERITAGE OBSERVATORY MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
# SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT
# NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
# A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  HERITAGE OBSERVATORY SHALL
# NOT BE LIABLE FOR ANY LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A
# RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
# DERIVATIVES.

import csv
import re

from majormode.perseus.model.enum import Enum
from majormode.perseus.model.locale import Locale
from majormode.perseus.model.geolocation import GeoPoint
from majormode.perseus.utils import cast


class Document(object):
    """
    Represent the attributes of a postcard or an old photograph.
    """
    SceneType = Enum(
        # Any form of life such as plant or living creature, would it be human
        # or any other animal.
        'being',

        # A place of cultural, historical, or natural significance for a group
        # or society.
        'heritage',

        # Any form of landscapes that show little or no human activity and are
        # created in the pursuit of a pure, unsullied depiction of nature.
        'scenery',

        # Any other type that doesn't represent a photograph, such as painting,
        # illustration, etc.
        'other',
    )

    # List of the tokens that can be used to modify a date.
    #
    # @warning: the longest tokens MUST be declared first, as the function
    #     ``_check_single_date_expression`` requires it:
    #
    #    for token in Document.DATE_MODIFIER_TOKENS:
    #        if date.startswith(token):
    DATE_MODIFIER_TOKENS = [ '<=', '>=', '~', '<', '>' ]

    # Regular expression to mach the expression of a simple date such as:
    #
    #     YYYY | YYYx | YYxx | YYYY-MM | YYYY-MM-DD
    SIMPLE_DATE_REGEX = re.compile(r'\d{4}|\d{3}x|\d{2}x{2}|\d{4}-\d{2}|\d{4}-\d{2}-\d{2}')


    def __init__(
            self,
            inventory_number,
            titles,
            author_name=None,
            collection_name=None,
            date=None,
            file_name=None,
            image_url=None,
            location_names=None,
            point=None,
            notes=None,
            scene_type=None,
            is_royalty_free=True):
        """
        Build a new instance ``Document``.


        @param inventory_number: inventory number of the photograph as defined
            by the organisation that is entrusted this photograph.

        @param titles: a tuple or a list of tuples corresponding to the
            caption of the photograph, written in a specified locale::

                (title:string, locale:Locale)

        @param author_name: name of the photographer.

        @param collection_name: name of the collection this photograph belongs to.

        @param file_name: file name of the photograph's image.  If a file name
             is not defined, it is built after the inventory name of the
             document.

        @param image_url: Uniform Resource Locator of the photograph’s image,
            if the image is shared online by the organisation that is
            entrusted this photograph.

        @param point: an instance containing the following attributes:

            * ``longitude``: longitude-angular distance, expressed in decimal
              degrees (WGS84 datum), measured from the center of the Earth, of a
              point east or west of the Prime Meridian.

            * ``latitude``: latitude-angular distance, expressed in decimal degrees
              (WGS84 datum), measured from the center of the Earth, of a point north
              or south of the Equator.

        @param location_names: list of names of the subdivisions of the
            location where the photograph has been taken.

        @param notes: a tuple or a list of tuples corresponding to the
            description of the photograph, written in a specified locale::

                (title:string, locale:Locale)

        @param date: precise or approximate date when this photograph has been
            taken.  This date MUST respect the specification ``time_spec``
            defined with the following Augmented Backus–Naur Form (ABNF)::

                time = YYYY | YYYx | YYxx | YYYY-MM | YYYY-MM-DD
                time_uncertainty = ?
                time_def = time | time time_uncertainty

                time_modifier = ~ | < | > | <= | >=
                time_expr = time_def | time_modifier time_def

                time_spec = time_expr | [time_expr, time_expr]

        @param scene_type: an item of ``SceneType`` that indicates the
            category of the subject taken in photo.

        @param is_royalty_free: indicate whether the photograph can be freely
            used.  Royalty free, or RF, refers to the right to use copyrighted
            material or intellectual property without the need to pay
            royalties or license fees for each use or per volume sold, or some
            time period of use or sales.
        """
        self.inventory_number = inventory_number
        assert inventory_number, 'No inventory number is provided'

        self.titles = titles and [ (content.strip(), locale if isinstance(locale, Locale) else Locale(locale))
                for (content, locale) in (titles if isinstance(titles, (list, set)) else [ titles ])
                    if content and len(content.strip()) ]
        assert self._check_i10n_labels(self.titles), 'Invalid titles format expression'

        self.author_name = author_name and author_name.strip()
        self.collection_name = collection_name and collection_name.strip()

        if not self._check_date_expression(date):
            print('[ERROR] Invalid date format expression "%s"' % date)
            date = None

        self.date = date

        self.file_name = file_name or '%s.jpg' % self.inventory_number

        self.image_url = image_url

        self.location_names = location_names and (location_names if isinstance(location_names, (list, set, tuple)) else [ location_names ])
        self.point = point

        self.notes = notes and [ (content.strip(), locale if isinstance(locale, Locale) else Locale(locale))
                for (content, locale) in (notes if isinstance(notes, (list, set)) else [ notes ])
                    if len(content.strip()) ]
        assert self._check_i10n_labels(self.notes), 'Invalid notes format expression'

        if scene_type:
            assert scene_type in Document.SceneType, 'The scene type MUST be an item of ``SceneType``'
        self.scene_type = scene_type

        self.is_royalty_free = is_royalty_free


    @staticmethod
    def _check_single_date_expression(date):
        """
        Check whether the given date expression respects the format
        specification ``time_expr``::

            time = YYYY | YYYx | YYxx | YYYY-MM | YYYY-MM-DD
            time_uncertainty = ?
            time_def = time | time time_uncertainty

            time_modifier = ~ | < | > | <= | >=
            time_expr = time_def | time_modifier time_def


        @param date: the string representation of a single date expression.


        @return: ``True`` if the given date respects the format specification
            of a single date; ``False`` otherwise.
        """
        if not date or len(date) == 0:
            return False

        # Check whether the date is prefixed with a modifier token.
        date_modifier = None
        for token in Document.DATE_MODIFIER_TOKENS:
            if date.startswith(token):
                date_modifier = token
                date = date[len(date_modifier):]
                if len(date) == 0:
                    return False
                break

        # Check whether the date is postfixed with the uncertainty token.
        is_date_uncertain = date[len(date) - 1] == '?'
        if is_date_uncertain:
            date = date[:len(date) - 1]
            if len(date) == 0:
                return False

        return Document.SIMPLE_DATE_REGEX.match(date)


    @staticmethod
    def _check_date_expression(date):
        """
        Check whether the given date expression respects the format
        specification ``time_spec``::

            time = YYYY | YYYx | YYxx | YYYY-MM | YYYY-MM-DD
            time_uncertainty = ?
            time_def = time | time time_uncertainty

            time_modifier = ~ | < | > | <= | >=
            time_expr = time_def | time_modifier time_def

            time_spec = time_expr | [time_expr, time_expr]


        @param date: a string representation of a date expression.


        @return: ``True`` if the given date respects the format specification
            of a date range or a single date; ``False`` otherwise.
        """
        if not date or len(date) == 0:
            return True

        # Check if the date expression corresponds to a date range.
        if date[0] == '[':
            if date[len(date) - 1] != ']':
                return False

            dates = [ _.strip() for _ in date[1:-1].split(',') ]
            if len(dates) != 2:
                return False

            [ date1, date2 ] = dates

            is_expression_valid = Document._check_single_date_expression(date1) and \
                    Document._check_single_date_expression(date2)

        else:
            is_expression_valid = Document._check_single_date_expression(date)

        return is_expression_valid


    @staticmethod
    def _check_i10n_labels(labels):
        """
        Check whether the given labels have all a locale defined.


        @param labels: a list of tuples ``(text, locale)`` where ``text`` is
            the string content of the label and ``locale`` is an instance
            ``Locale``.


        @return: ``True`` if the specified labels have all a locale defined;
            ``False`` otherwise.
        """
        return labels is None or (
                isinstance(labels, list) and \
                all([ isinstance(locale, Locale) for (_, locale) in labels ]))


class CommaSeparatedValueDocument(Document):
    # Define the fields used in a comma-separated value file that declare
    # documents.
    CSV_FIELD_NAME_INVENTORY_NUMBER = '#'
    CSV_FIELD_NAME_FILE_NAME = 'file'
    CSV_FIELD_NAME_IMAGE_URL = 'url'
    CSV_FIELD_NAME_TITLE = re.compile(r'title_([a-z]{3}){1}')
    CSV_FIELD_NAME_TITLE_PREFIX = 'title_'
    CSV_FIELD_NAME_NOTE = re.compile(r'note_([a-z]{3}){1}')
    CSV_FIELD_NAME_NOTE_PREFIX = 'note_'
    CSV_FIELD_NAME_AUTHOR_NAME = 'author'
    CSV_FIELD_NAME_COLLECTION_NAME = 'collection'
    CSV_FIELD_NAME_LOCATION_NAMES = 'location'
    CSV_FIELD_NAME_POINT = 'point'
    CSV_FIELD_NAME_SCENE_TYPE = 'scene'
    CSV_FIELD_NAME_DATE = 'date'
    CSV_FIELD_NAME_IS_ROYALTY_FREE = 'royalty_free'

    # Define the list of fields that are mandatory and those that are
    # optional.
    CSV_FIELDS = [
        (CSV_FIELD_NAME_INVENTORY_NUMBER, True),
        (CSV_FIELD_NAME_FILE_NAME, False),
        (CSV_FIELD_NAME_IMAGE_URL, False),
        (CSV_FIELD_NAME_TITLE, True),
        (CSV_FIELD_NAME_NOTE, False),
        (CSV_FIELD_NAME_AUTHOR_NAME, False),
        (CSV_FIELD_NAME_COLLECTION_NAME, False),
        (CSV_FIELD_NAME_LOCATION_NAMES, False),
        (CSV_FIELD_NAME_POINT, False),
        (CSV_FIELD_NAME_SCENE_TYPE, False),
        (CSV_FIELD_NAME_DATE, False),
        (CSV_FIELD_NAME_IS_ROYALTY_FREE, False)
    ]

    # Declare the list of field names.
    CSV_FIELD_NAMES = [ name for (name, is_required) in CSV_FIELDS ]

    # Declare the list of the names of the mandatory fields.
    CSV_REQUIRED_FIELD_NAMES = [ name for (name, is_required) in CSV_FIELDS if is_required ]


class CommaSeparatedValueDocumentReader(object):
    REGEX_GEOGRAPHIC_COORDINATES = re.compile(r'\(\s*(\d*(?:\.\d+)?)\s*,\s*((\d*(?:\.\d+)?))\s*\)')

    def __init__(self, file_path_name,
            delimiter='\t',
            quotechar='"'):
        with open(file_path_name, 'rt') as csv_file:
            reader = csv.DictReader(csv_file, delimiter=delimiter, quotechar=quotechar)
            self.rows = [ row for row in reader ]

        self.current = 0

    def __iter__(self):
        return self

    def __next__(self):
        self.next()

    @classmethod
    def __build_document(cls, row):
        titles = [ (row[match.group(0)], match.group(1))
            for match in [ CommaSeparatedValueDocument.CSV_FIELD_NAME_TITLE.match(field_name) for field_name in row ]
                if match ]

        notes = [ (row[match.group(0)], match.group(1))
            for match in [ CommaSeparatedValueDocument.CSV_FIELD_NAME_NOTE.match(field_name) for field_name in row ]
                if match ]

        geographic_coordinate_expression = row.get(CommaSeparatedValueDocument.CSV_FIELD_NAME_POINT)
        match = geographic_coordinate_expression and cls.REGEX_GEOGRAPHIC_COORDINATES.match(geographic_coordinate_expression)
        point = match and GeoPoint(match.group(1), match.group(2))

        return Document(
            row[CommaSeparatedValueDocument.CSV_FIELD_NAME_INVENTORY_NUMBER],
            titles,
            author_name=row.get(CommaSeparatedValueDocument.CSV_FIELD_NAME_AUTHOR_NAME),
            collection_name=row.get(CommaSeparatedValueDocument.CSV_FIELD_NAME_COLLECTION_NAME),
            date=row.get(CommaSeparatedValueDocument.CSV_FIELD_NAME_DATE),
            file_name=row.get(CommaSeparatedValueDocument.CSV_FIELD_NAME_FILE_NAME),
            image_url=row.get(CommaSeparatedValueDocument.CSV_FIELD_NAME_IMAGE_URL),
            location_names=row.get(CommaSeparatedValueDocument.CSV_FIELD_NAME_LOCATION_NAMES),
            point=point,
            notes=notes,
            scene_type=cast.string_to_enum(row.get(CommaSeparatedValueDocument.CSV_FIELD_NAME_SCENE_TYPE), Document.SceneType, strict=False),
            is_royalty_free=cast.string_to_boolean(row.get(CommaSeparatedValueDocument.CSV_FIELD_NAME_IS_ROYALTY_FREE)))

    def next(self):
        if self.current >= len(self.rows):
            raise StopIteration
        else:
            self.current += 1
            return self.__build_document(self.rows[self.current - 1])




# Copyright (C) 2019 Heritage Observatory.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Heritage Observatory or one of its subsidiaries.  You shall not
# disclose this confidential information and shall use it only in
# accordance with the terms of the license agreement or other applicable
# agreement you entered into with Heritage Observatory.
#
# HERITAGE OBSERVATORY MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
# SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT
# NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
# A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  HERITAGE OBSERVATORY SHALL
# NOT BE LIABLE FOR ANY LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A
# RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
# DERIVATIVES.

import uuid

from majormode.perseus.constant.contact import ContactName
from majormode.perseus.constant.place import AddressComponentType
from majormode.perseus.constant.privacy import Visibility
from majormode.perseus.model.contact import Contact
from majormode.perseus.model.geolocation import GeoPoint
from majormode.perseus.model.label import Label
from majormode.perseus.model.locale import DEFAULT_LOCALE
from majormode.perseus.model.place import Place
from majormode.perseus.utils import cast
import numpy


class HeritageSite(object):
    def __init__(
            self,
            category,
            location,
            site_id=None,
            area_id=None,
            address=None,
            contacts=None,
            locale=DEFAULT_LOCALE,
            timezone=None):
        """

        @param category: an item of the enumeration ``SportCategory``
            representing the sport played at this site.

        @param location: either an instance ``GeoPoint`` corresponding to
            the geographic coordinates of the location, either a polygon
            specifying the geofence's region of the site as an array of
            vertices.  There must be at least three vertices.  Each vertex
            is a tuple composed of a longitude, a latitude, and a altitude::

                [ longitude, latitude, altitude ]
            where:

            * ``altitude`` (optional): altitude in meters of the site.

            * ``latitude`` (required): latitude-angular distance, expressed
              in decimal degrees (WGS84 datum), measured from the center of the
              Earth, of a point north or south of the Equator corresponding to
              the site's location.

            * ``longitude`` (required): longitude-angular distance,
              expressed in decimal degrees (WGS84 datum), measured from the
              center of the Earth, of a point east or west of the Prime
              Meridian corresponding to the site's location.

            Note that the first and last vertices must not be identical; a
            polygon is "auto-closed" between the first and last vertices.

            When specifying the geofence of the site, the function
            automatically computes the coordinates of the geometric center
           (centroid) of the polygon representing the geofence of the sport
            site.  It corresponds to the arithmetic mean ("average")
           position of all the points in the shape.

        @param site_id: the identification of the site when this site
            is already registered against the server platform.

        @param area_id: identification of the geographic area where the
            site is located in.  This parameter is optional if the argument
            ``address`` is passed.  Both ``area_id`` and ``address`` can be
            passed to the constructor.  If this argument is passed to the
            constructor, it takes precedence over any administrative
            division of the same level or larger that would be defined as
            part of the address components.

        @param address:postal address of the site, composed of one or more
            address components, which textual information is written in the
            specified locale.  An address component is defined with a
            component type and its value.  The component type is one of these:

            * ``city``

            * ``country``

            * ``district``: represent districts in rural areas and
              precincts in urban areas.

            * ``hamlet``: small settlement, with a small population, in
              a rural area, or a # component of a larger settlement or
              municipality.

            * ``house_number``: unique number of the  site in the
              street or area, which eases to locate this particular site.
              House numbering schemes vary by site, and in many cases even
              within cities.  In some areas of the world, including many
              remote areas, houses are not numbered at all, instead simply
              being named.  In some other areas, this numbering can be
              composed of a first number along the street followed by a
              second the number along intersecting street, or they would
              adopt a system where the city is divided into small sections
              each with its own numeric code.  The houses within that zone
              are then labeled based on the order in which they were
              constructed, or clockwise around the block.

            * ``neighborhood``: a geographically localised community
              within a larger city, town, # suburb or rural area.
              Neighborhoods are often social communities  with considerable
              face-to-face interaction among members.  Researchers have not
              agreed on an exact definition.  Neighborhood is  generally
              defined spatially as a specific geographic area and
              functionally as a set of social networks.

            * ``postal_code``: postal code as used to address postal
              mail within the country.

            * ``province``

            * ``recipient_name``: intended recipients name or other
              designation, which can be an individual, a business, a site,
              an organization.

            * ``street_name``: street name or odonym, i.e., an
              identifying name, given to the street where the site is
              located in.  The street name usually forms part of the address
              (though addresses in some parts of the world, notably most of
              Japan, make no reference to street names).

            * ``ward``: represent rural communes, commune-level town,
              and urban wards.

        @param contacts: list of properties such as e-mail addresses, phone
            numbers, etc., in respect of the electronic business card
            specification (vCard).  The contact information is represented
            by a list of tuples of the following form::

                [ (name:string, value:string, is_primary:boolean), ... ]

            where:

            * ``name`` (required): type of this contact information, which
              can be one of these standard names in respect of the electronic
              business card specification (vCard):

              * ``FN``: formatted name, i.e., the way that the name is to
                be displayed.

              * ``EMAIL``: e-mail address.

              * ``PHONE``: phone number in E.164 numbering plan, an ITU-T
                recommendation which defines the international public
                telecommunication numbering plan used in the Public Switched
                Telephone Network (PSTN).

              * ``WEBSITE``: Uniform Resource Locator (URL) of a Web site.

            * ``value`` (required): value of this contact information
              representing by a string, such as ``+84.01272170781``, the
              formatted value for a telephone number property.

            * ``is_primary`` (optional): indicate whether this contact
              information is the primary for this site.  By default, the first
              contact information of a given type is the primary of this site.

        @param locale: an instance ``Locale`` of the textual information
            that describes this site.

        @param timezone: time zone at the site's location.  It is the
            difference between the time at this location and UTC
            (Coordinated Universal Time).  UTC is also known as GMT or
            Greenwich Mean Time or Zulu Time.
        """
        self.category = category

        if isinstance(location, GeoPoint):
            self.location = location
            self.geofence = None
        else:
            if len(location) < 3:
                raise ValueError("The polygon of the site's geofence MUST be defined with at least three vertices")

            # Compute the coordinates of the geometric center (centroid) of the
            # polygon representing the geofence of the sport site.  It
            # corresponds to the arithmetic mean ("average") position of all the
            # points in the shape.
            (longitude, latitude, altitude) = numpy.mean(location, axis=0)
            self.location = GeoPoint(longitude, latitude, altitude=altitude)
            self.geofence = location

        self.site_id = site_id

        self.area_id = area_id
        self.address = address
        self.contacts = contacts

        self.locale = locale
        self.timezone = timezone


    @staticmethod
    def from_json(payload):
        """
        Build a ``site`` instance from the specified JSON object.

        @param payload: JSON representation of a site::

                {
                  "area_id": string,
                  "address": {
                    component_type: string,
                    ...
                  }
                  "category": string,
                  "contacts": [
                    [ name:string, value:string, is_primary:boolean ],
                    ...
                  ],
                  "geofence": [ vertex, ... ],
                  "locale": string,
                  "location": coordinates,
                  "timezone": integer
                }

            where:

            * ``area_id`` (optional): identification of the geographic area
              where the site is located in.  This parameter is optional if the
              parameter ``address`` is passed.  Both ``area_id`` and ``address``
              can be passed to the function.  If this parameter is passed to the
              function, it takes precedence over any administrative division of
              the same level or larger that would be defined as part of the
              address components.

            * ``address`` (required): postal address of the site, composed of
              one or more address components, which textual information is
              written in the specified locale.  An address component is defined
              with a component type and its value.  The component type is one of
              these:

                * ``city``

                * ``country``

                * ``district``: represent districts in rural areas and
                  precincts in urban areas.

                * ``hamlet``: small settlement, with a small population, in
                  a rural area, or a # component of a larger settlement or
                  municipality.

                * ``house_number``: unique number of the  site in the
                  street or area, which eases to locate this particular site.
                  House numbering schemes vary by site, and in many cases even
                  within cities.  In some areas of the world, including many
                  remote areas, houses are not numbered at all, instead simply
                  being named.  In some other areas, this numbering can be
                  composed of a first number along the street followed by a
                  second the number along intersecting street, or they would
                  adopt a system where the city is divided into small sections
                  each with its own numeric code.  The houses within that zone
                  are then labeled based on the order in which they were
                  constructed, or clockwise around the block.

                * ``neighborhood``: a geographically localised community
                  within a larger city, town, # suburb or rural area.
                  Neighborhoods are often social communities  with considerable
                  face-to-face interaction among members.  Researchers have not
                  agreed on an exact definition.  Neighborhood is  generally
                  defined spatially as a specific geographic area and
                  functionally as a set of social networks.

                * ``postal_code``: postal code as used to address postal
                  mail within the country.

                * ``province``

                * ``recipient_name``: intended recipients name or other
                  designation, which can be an individual, a business, a site,
                  an organization.

                * ``street_name``: street name or odonym, i.e., an
                  identifying name, given to the street where the site is
                  located in.  The street name usually forms part of the address
                  (though addresses in some parts of the world, notably most of
                  Japan, make no reference to street names).

                * ``ward``: represent rural communes, commune-level town,
                  and urban wards.

            * ``category_id`` (optional): category qualifying this site.

            * ``contacts`` (optional): list of properties such as e-mail
              addresses, phone numbers, etc., in respect of the electronic
              business card specification (vCard).  The contact information is
              represented by a list of tuples of the following form::

                  [ [ name:string, value:string, is_primary:boolean ], ... ]

              where:

              * ``name`` (required): type of this contact information, which
                can be one of these standard names in respect of the electronic
                business card specification (vCard):

                * ``FN``: formatted name, i.e., the way that the name is to
                  be displayed.

                * ``EMAIL``: e-mail address.

                * ``PHONE``: phone number in E.164 numbering plan, an ITU-T
                  recommendation which defines the international public
                  telecommunication numbering plan used in the Public Switched
                  Telephone Network (PSTN).

                * ``WEBSITE``: Uniform Resource Locator (URL) of a Web site.

              * ``value`` (required): value of this contact information
                representing by a string, such as ``+84.01272170781``, the
                formatted value for a telephone number property.

              * ``is_primary`` (optional): indicate whether this contact
                information is the primary for this site.  By default, the first
                contact information of a given type is the primary of this site.

            * ``geofence`` (optional): a polygon specifying the geofence's
              region of this site.  It corresponds to an array of vertices.
              There must be at least three vertices. Each vertex is a tuple
              composed of a longitude, a latitude, and a altitude::

                [ longitude, latitude, altitude ]

              Note that the first and last vertices must not be identical; a
              polygon is "auto-closed" between the first and last vertices.

            * ``locale`` (required): locale of the textual information that
              describes this site.  A locale corresponds to a tag respecting RFC
              4646, i.e., a ISO 639-3 alpha-3 code element optionally followed by
              a dash character ``-`` and a ISO 3166-1 alpha-2 code (referencing
              the country that this language might be specific to).  For example:
              ``eng`` (which denotes a standard English), ``eng-US`` (which
              denotes an American English).

            * ``location`` (optional): geographic coordinates of the location of
              the site represented with the following JSON structure::

                  {
                    "accuracy": decimal
                    "altitude": decimal,
                    "latitude": decimal,
                    "longitude": decimal,
                  }

              where:

              * ``accuracy`` (optional): accuracy of the site's position in
                meters.

              * ``altitude`` (optional): altitude in meters of the site.

              * ``latitude`` (required): latitude-angular distance, expressed
                in decimal degrees (WGS84 datum), measured from the center of the
                Earth, of a point north or south of the Equator corresponding to
                the site's location.

              * ``longitude`` (required): longitude-angular distance,
                expressed in decimal degrees (WGS84 datum), measured from the
                center of the Earth, of a point east or west of the Prime
                Meridian corresponding to the site's location.

                .. note::

                   The parameter ``location`` is ignored when the parameter
                   ``geofence`` is provided.  The platform computes the coordinates
                   of the geometric center (centroid) of the polygon representing the
                   geofence of the sport site.  It corresponds to the arithmetic
                   mean ("average") position of all the points in the shape.

            * ``timezone`` (required): time zone at the site's location.  It is
              the difference between the time at this location and UTC
              (Coordinated Universal Time).  UTC is also known as GMT or
              Greenwich Mean Time or Zulu Time.

        @note: the name of the site corresponds to the address component
            ``recipient_name``.

        @return: a ``site`` instance or ``None`` if the JSON payload is
            nil.
        """
        return payload and \
            Place(
                SiteMetadataClass.from_json(payload.get('category')),
                location=[
                    (float(lon), float(lat), float(alt))
                    for (lon, lat, alt) in payload['geofence']
                ] if payload.get('geofence')
                    else GeoPoint.from_json(payload['location']),
                site_id=cast.string_to_uuid(payload.get('site_id')),
                area_id=cast.string_to_uuid(payload.get('area_id')),
                address=payload.get('address') and dict([
                    (cast.string_to_enum(name, AddressComponentType), value)
                    for (name, value) in payload['address'].iteritems()
                ]),
                contacts=payload.get('contact') and [
                    Contact(cast.string_to_enum(contact[0], ContactName), contact[1], contact[2] if len(contact) == 3 else False)
                    for contact in payload['contacts']
                ],
                locale=cast.string_to_locale(payload.get('locale')) and DEFAULT_LOCALE,
                timezone=payload.get('timezone'))

    def is_info_complete(self):
        """
        Indicate if the information of the site could be considered complete
        or not, i.e., whether it has a name.

        @return: ``True`` if the site's information is complete, ``False``
            otherwise.
        """
        return self.address is not None and \
               self.address.get(AddressComponentType.recipient_name) is not None


class SiteCategory(object):
    """
    MetadataClass that indicates the particular use of heritage site.
    """
    def __init__(self,
            category_id=None,
            labels=None):
        """
        Build an instance ``SiteCategory``.

        @param account_id:  identification of the account
              of the user who submitted this localized name of the category.

        @param category_id: identification of the category.

        @param labels: localized humanely-readable names of this category.  A
            caption is provided by a client application when the category has
            not been already registered to the platform, i.e., the user enters
            a new category.

            * ``account_id`` (required): identification of the account
              of the user who submitted this localized name of the category.

            * ``caption`` (required): humanely-readable names of this
              category in a given locale.

            * ``locale`` (required): locale used for the caption of this
              category is return.

            * ``moderator_id`` (optional): identification of the account
              of a moderator of the platform who reviewed this particular
              localization of the category.

            * ``object_status`` (required): current status of this
              localized caption, either pending, either reviewed.

        @param moderator_id: identification of the account of a moderator of
            the platform who reviewed this localized caption of the category.
        """
        if not (category_id or labels):
            raise ValueError("MetadataClass of heritage site MUST be defined either by an identification or a localized caption")

        self.is_new = category_id is None
        self.category_id = category_id or uuid.uuid1()
        self.labels = labels

    @property
    def label(self):
        if len(self.labels) != 1:
            raise ValueError('This category has no localized caption or more than one localized caption defined')

        return self.labels[0]

    @staticmethod
    def from_json(payload, locale=None):
        if not payload:
            return None

        category_id = cast.string_to_uuid(payload.get('category_id'), strict=False)

        labels_payload = payload.get('labels')
        if labels_payload:
            labels = []
            for label_payload in labels_payload:
                label = Label.from_json(label_payload)
                if locale:
                    label.locale = locale

                label.account_id = cast.string_to_uuid(label_payload.get('account_id'))
                label.moderator_id = cast.string_to_uuid(label_payload.get('moderator_id'))

                labels.append(label)

        else:
            labels = None if category_id else [ Label(payload['caption'], locale) ]

        return SiteCategory(
                category_id=cast.string_to_uuid(payload.get('category_id')),
                labels=labels)


class SiteMetadataClass(object):
    """
    Matadatum class that is used to describe more specifically heritage
    sites.  Metadata provide information about one or more aspects of
    buildings such as, for instance, the number of floors, date of
    construction, date of completion, architectural style, etc.
    """
    def __init__(
            self,
            class_id=None,
            labels=None,
            visibility=Visibility.public,
            team_ids=None):
        """
        Build an instance ``SiteMetadataClass``.

        @param account_id:  identification of the account of the user who
            submitted this localized name of the class.

        @param class_id: identification of the class.

        @param labels: localized humanely-readable names of this class.  A
            caption is provided by a client application when the class has
            not been already registered to the platform, i.e., the user enters
            a new class.

            * ``account_id`` (required): identification of the account
              of the user who submitted this localized name of the class.

            * ``caption`` (required): human-readable names of this
              class in a given locale.

            * ``locale`` (required): locale used for the caption of this
              class is return.

            * ``moderator_id`` (optional): identification of the account
              of a moderator of the platform who reviewed this particular
              localization of the class.

            * ``object_status`` (required): current status of this
              localized caption, either pending, either reviewed.

        @param moderator_id: identification of the account of a moderator of
            the platform who reviewed this localized caption of the class.

        @param visibility: an item of the enumeration ``Visibility`` that
            indicates which visibility needs to be setup for this metadatum
            class.

        @param team_ids: list of identifications of the teams that are allowed
            to view metadata of this class, when the visibility is set to
            ``Visibility.organization``.
        """
        if not (class_id or labels):
            raise ValueError("Metadatum class of heritage site MUST be defined either by an identification or a localized caption")

        self.is_new = class_id is None
        self.class_id = class_id or uuid.uuid1()
        self.labels = labels
        self.team_ids = team_ids
        self.visibility = visibility


    @staticmethod
    def from_json(payload, locale=None):
        """
        Build an instance ``SiteMetadataClass`` from a JSON expression.

        @param payload: a JSON expression of the following structure:

                {
                  "class_id": string,
                  "labels": [
                    {
                      "account_id": string,
                      "caption": string,
                      "locale": string,
                      "moderator_id": string,
                      "object_status": integer
                    },
                    ...
                  ],
                  "team_ids": [ string, ... ],
                  "visibility": string
                }

            where:

            * ``class_id`` (required): identification of the class.

            * ``labels`` (required): localized human-readable names of this
              class.  If the class has not a caption in the specified locale,
              the platform returns all the available localized captions of
              this class.

              * ``account_id`` (required): identification of the account of
                the user who submitted this localized name of the class.

              * ``caption`` (required): human-readable names of this class in
                a given locale.

              * ``locale`` (required): locale used for the caption of this
                class is return.  A locale corresponds to a tag respecting
                RFC 4646, i.e., a ISO 639-3 alpha-3 code element optionally
                followed by a dash character ``-`` and a ISO 3166-1 alpha-2
                code (referencing the country that this language might be
                specific to).  For example: ``eng`` (which denotes a standard
                English), ``eng-US`` (which denotes an American English).

              * ``moderator_id`` (optional): identification of the account of
                a moderator of the platform who reviewed this particular
                localization of the class.

              * ``object_status`` (required): current status of this localized
                caption, either pending, either reviewed.

            * ``team_ids`` (optional): list of identifications of the teams
              that are allowed to view this metadatum.

            * ``visibility`` (optional): indicate which visibility needs to be
              setup for these metadatum:

              * ``organization``: the metadatum is restricted to the
                specified organisations (cf. the argument list composed of
                ``team_id``). Only the members of these organisations, and the
                moderators of the platform, can view it.

              * ``private``: the metadatum is private.  Only the user who
                added this metadatum, and the moderators of the platform, can
                view.

              * ``public``: the metadatum is public.  Anyone registered to
                the platform can view it.

        @param locale: an instance ``Locale`` that indicates the language in
            which the caption of the class is written.  If the JSON
            attribute ``locale`` is defined, it overrides this argument
            ``locale``.

        @return: an instance ``SiteMetadataClass``.
        """
        if not payload:
            return None

        labels = []
        for label_payload in payload['labels']:
            label = Label.from_json(label_payload)
            if locale:
                label.locale = locale

            label.account_id = cast.string_to_uuid(label_payload.get('account_id'))
            label.moderator_id = cast.string_to_uuid(label_payload.get('moderator_id'))

            labels.append(label)

        return SiteMetadataClass(
            class_id=cast.string_to_uuid(payload.get('class_id')),
            labels=labels,
            team_ids=payload.get('team_ids') and [
                cast.string_to_uuid(team_id)
                for team_id in payload['team_ids']
            ],
            visibility=cast.string_to_enum(payload.get('visibility'), Visibility, strict=False) and Visibility.public)



# Copyright (C) 2019 Heritage Observatory.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Heritage Observatory or one of its subsidiaries.  You shall not
# disclose this confidential information and shall use it only in
# accordance with the terms of the license agreement or other applicable
# agreement you entered into with Heritage Observatory.
#
# HERITAGE OBSERVATORY MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
# SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT
# NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
# A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  HERITAGE OBSERVATORY SHALL
# NOT BE LIABLE FOR ANY LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A
# RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
# DERIVATIVES.

from majormode.perseus.model.enum import Enum


AlertFrequency = Enum(
    # Indicate that the user wants to receive alerts as soon as they
    # happen.  This is the default option.
    'immediate',

    # Indicate that the user wants to receive alerts at most once a day.
    'daily',

    # Indicate that the user wants to receive alerts at most once a week.
    'weekly'
)



# Copyright (C) 2019 Heritage Observatory.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Heritage Observatory or one of its subsidiaries.  You shall not
# disclose this confidential information and shall use it only in
# accordance with the terms of the license agreement or other applicable
# agreement you entered into with Heritage Observatory.
#
# HERITAGE OBSERVATORY MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
# SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT
# NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
# A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  HERITAGE OBSERVATORY SHALL
# NOT BE LIABLE FOR ANY LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A
# RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
# DERIVATIVES.

from majormode.perseus.constant.obj import ObjectStatus
from majormode.perseus.model.enum import Enum


# Extended statuses of a heritage at risk report.
ReportObjectStatus = ObjectStatus.extend(
    # The report has been created by a user but not submitted yet for
    # approval by reviewers.
    'draft',

    # The report has been submitted for approval by reviewers, but it is
    # still not accepted or rejected.
    'submitted',

)

# Actions that a reviewer can select related to a heritage at risk
# report submitted by a user.
ReportReviewAction = Enum(
    # The reviewer approves the heritage at risk report submitted by a user.
    'accept',

    # The reviewer has not idea if the heritage at risk report submitted by
    # a user is valid or fake.
    'dunno',

    # The reviewer rejects the heritage at risk report submitted by a user.
    'reject',
)



# Copyright (C) 2019 Heritage Observatory.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Heritage Observatory or one of its subsidiaries.  You shall not
# disclose this confidential information and shall use it only in
# accordance with the terms of the license agreement or other applicable
# agreement you entered into with Heritage Observatory.
#
# HERITAGE OBSERVATORY MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
# SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT
# NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
# A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  HERITAGE OBSERVATORY SHALL
# NOT BE LIABLE FOR ANY LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A
# RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
# DERIVATIVES.

from majormode.perseus.constant.obj import ObjectStatus
from majormode.perseus.model.enum import Enum


# Types of orientation of a photo.
Orientation = Enum(
    # The display is wider than it is tall.
    'landscape',

    # The display is taller than it is wide.
    'portrait'
)

# Type of hardware that has generated the image of a photo, either a
# scanner, either a DSLR/smartphone camera.
PhotoProvider = Enum(
    # Define a Digital Single-Lens Reflex (DSLR) camera or a handheld
    # personal computer (smartphone or tablet).
    'camera',

    # Define a device that optically scans photograph printed on paper, such
    # as a flatbed or hand-held scanner,
    'scanner'
)

# Extended status of a photo.
PhotoObjectStatus = ObjectStatus.extend(
    'processing',
    'dmz'
)

# Types of the scene depicted on a photo.  This classification allows
# the platform to determine whether a particular photo could be
# precisely geolocated using a mechanical turk method.  This applies to
# photos of heritage sites.
SceneType = Enum(
    # Any form of life such as plant or living creature, would it be human
    # or any other animal.
    'being',

    # A place of cultural, historical, or natural significance for a group
    # or society.
    'heritage',

    # Any other type of image that doesn't represent a photograph, such as
    # painting, illustration, etc.
    'other',

    # Any form of landscapes that show little or no human activity and are
    # created in the pursuit of a pure, unsullied depiction of nature, also
    # known as scenery.
    'scenery',
)



# Copyright (C) 2019 Heritage Observatory.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Heritage Observatory or one of its subsidiaries.  You shall not
# disclose this confidential information and shall use it only in
# accordance with the terms of the license agreement or other applicable
# agreement you entered into with Heritage Observatory.
#
# HERITAGE OBSERVATORY MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
# SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT
# NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
# A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  HERITAGE OBSERVATORY SHALL
# NOT BE LIABLE FOR ANY LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A
# RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
# DERIVATIVES.

from majormode.perseus.constant.obj import ObjectStatus
from majormode.perseus.utils import cast

from heobs.constant.alert import AlertFrequency


class AreaSubscriptionSync(object):
    """
    Represent the subscription of a user to alerts from a geographic area
    to be synchronized with the server platform.
    """
    def __init__(
            self,
            area_id,
            object_status,
            frequency=None):
        """
        Build an instance ``AreaAlertSubscriptionSync`` used to synchronize
        the subscription of a user to alerts from a geographic area.


        @param area_id: identification of a geographic area.
         
        @param object_status: current status of the subscription of the user to
            alerts from this geographic area:
            
            * ``ObjectStatus.pending``: the user is subscribing to alerts from
              this geographic area.
         
            * ``ObjectStatus.enabled``: the user has changed the properties of his
              subscription to alerts from this geographic area, i.e., the frequency
              at which alerts should be sent to him.

            * ``ObjectStatus.deleted``: the user is unsubscribing from alerts from
              this geographic area.
         
        @param frequency: an item of the enumeration ``AlertFrequency`` which
            indicates the frequency at which alerts related to this geographic
            area should be sent to the user.  This argument MUST be specified
            when the user subscribes to alerts from the geographic area.
        """
        self.area_id = area_id

        assert object_status in (ObjectStatus.pending, ObjectStatus.enabled, ObjectStatus.deleted), 'Invalid status of the subscription to synchronize'
        assert object_status == ObjectStatus.deleted or frequency is not None, 'The frequency argument MUST be passed'
        self.object_status = object_status

        self.frequency = frequency

    @staticmethod
    def from_json(payload):
        return payload and AreaSubscriptionSync(
            cast.string_to_uuid(payload['area_id']),
            cast.string_to_enum(payload['object_status'], ObjectStatus),
            frequency=cast.string_to_enum(payload.get('frequency'), AlertFrequency))



# Copyright (C) 2019 Heritage Observatory.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Heritage Observatory or one of its subsidiaries.  You shall not
# disclose this confidential information and shall use it only in
# accordance with the terms of the license agreement or other applicable
# agreement you entered into with Heritage Observatory.
#
# HERITAGE OBSERVATORY MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
# SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT
# NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
# A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  HERITAGE OBSERVATORY SHALL
# NOT BE LIABLE FOR ANY LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A
# RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
# DERIVATIVES.

import os

import pipfile
import setuptools


__author__ = "Daniel CAUNE"
__copyright__ = "Copyright (C) 2019, Heritage Observatory"
__credits__ = ["Daniel CAUNE"]
__email__ = "daniel.caune@heritageobservatory.org"
__license__ = "Copyright (C) 2019 Heritage Observatory. All rights reserved."
__maintainer__ = "Daniel CAUNE"
__status__ = "Production"
__version__ = '1.0.6'


# Base directory where this file is located.
BASE_DIR = os.path.dirname(__file__)


def get_requirements():
    pip_file = pipfile.load()
    return os.linesep.join([
        package_name
        for package_name, package_version in pip_file.data['default'].items()])


def read_file(file_path_name):
    with open(file_path_name, mode='rt', encoding='utf-8') as fd:
        return fd.read()


setuptools.setup(
    author=__author__,
    author_email=__email__,
    classifiers = [
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Development Status :: 5 - Production/Stable',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python'
    ],
    description='Heritage Observatory Core Python Library',
    install_requires=get_requirements(),
    license=__license__,
    long_description=read_file(os.path.join(BASE_DIR, 'README.md')),
    long_description_content_type='text/markdown',
    name='heobs-core-library',
    packages=setuptools.find_packages(),
    platforms=['any'],
    python_requires='>=3',
    version=__version__
)



# Copyright (C) 2019 Heritage Observatory.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Heritage Observatory or one of its subsidiaries.  You shall not
# disclose this confidential information and shall use it only in
# accordance with the terms of the license agreement or other applicable
# agreement you entered into with Heritage Observatory.
#
# HERITAGE OBSERVATORY MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
# SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT
# NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
# A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  HERITAGE OBSERVATORY SHALL
# NOT BE LIABLE FOR ANY LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A
# RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
# DERIVATIVES.

from majormode.perseus.model.obj import Object
from majormode.perseus.utils import cast

from heobs.constant.report import HeritageAtRiskActionPriority
from heobs.constant.report import HeritageCondition
from heobs.constant.report import HeritageDesignation
from heobs.constant.report import HeritageGrade
from heobs.constant.report import HeritageOccupancy
from heobs.constant.report import HeritageOwnerType


class HeritageAtRiskReport(Object):
    def __init__(
            self,
            action_priority=None,
            condition=None,
            designation=None,
            grade=None,
            occupancy=None,
            owner_type=None):
        super().__init__()
        self.action_priority = action_priority
        self.designation = designation
        self.condition = condition
        self.grade = grade
        self.occupancy = occupancy
        self.owner_type = owner_type

    @staticmethod
    def from_json(payload):
        return payload and HeritageAtRiskReport(
            action_priority=cast.string_to_enum(payload.get('action_priority'), HeritageAtRiskActionPriority),
            condition=cast.string_to_enum(payload.get('condition'), HeritageCondition),
            designation=cast.string_to_enum(payload.get('designation'), HeritageDesignation),
            grade=cast.string_to_enum(payload.get('grade'), HeritageGrade),
            occupancy=cast.string_to_enum(payload.get('occupancy'), HeritageOccupancy),
            owner_type=cast.string_to_enum(payload.get('owner_type'), HeritageOwnerType)
        )
